﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace InQuikModule
{
    public enum LUA_TYPE
    {
        LUA_TNONE               =-1,
        LUA_TNIL                = 0,
        LUA_TBOOLEAN            = 1,
        LUA_TLIGHTUSERDATA     = 2,
        LUA_TNUMBER             = 3,
        LUA_TSTRING             = 4,
        LUA_TTABLE              = 5,
        LUA_TFUNCTION           = 6,
        LUA_TUSERDATA           = 7,
        LUA_TTHREAD             = 8
    }

    public static class Lua
    {
        [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
        public static extern void OutputDebugString(string message);

        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I4)]
        public delegate int LuaNativeFunction(IntPtr L);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_tolstring")]
        public static extern IntPtr lua_tolstring(IntPtr luaState, int index, out uint strLen);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_pushcclosure(IntPtr L, LuaNativeFunction fn, int n);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_setfield(IntPtr L, int idx, [MarshalAs(UnmanagedType.LPStr)] string k);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_getfield(IntPtr L, int idx, [MarshalAs(UnmanagedType.LPStr)] string k);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_rawget")]
        public static extern void lua_rawget(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_tonumber")]
        public static extern double lua_tonumber(IntPtr luaState, int index);


        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_objlen")]
        public static extern int lua_objlen(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_next")]
        public static extern int lua_next(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_pushvalue")]
        public static extern void lua_pushvalue(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_pushnil")]
        public static extern void lua_pushnil(IntPtr luaState);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_pushstring(IntPtr L, [MarshalAs(UnmanagedType.LPStr)] string k);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_pushnumber")]
        public static extern void lua_pushnumber(IntPtr luaState, double number);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_pushinteger")]
        public static extern void lua_pushinteger(IntPtr luaState, int number);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_settop")]
        public static extern void lua_settop(IntPtr luaState, int newTop);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_gettop")]
        public static extern int lua_gettop(IntPtr luaState);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_remove")]
        public static extern void lua_remove(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_insert")]
        public static extern void lua_insert(IntPtr luaState, int newTop);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_call(IntPtr L, int nargs, int nresults);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "lua_type")]
        public static extern int lua_type(IntPtr luaState, int index);

        [DllImport("qlua.dll", CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Ansi)]
        public static extern int lua_pcall(IntPtr L, int nargs, int nresults, int errfunc);

        public static void lua_pop(IntPtr L, int idx)
        {
            Lua.lua_settop(L, Lua.lua_gettop(L) - idx);
            //Lua.lua_settop(L, -idx - 1);
        }

    }
}
